/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Model;

import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import org.xml.sax.SAXException;

/**
 *
 * @author Yannic Schnetz
 */
public class Controller extends HttpServlet {

    public static UserDatabase userDatabase = null;
    public static OfferDatabase offerDatabase = null;

    private boolean isInArray(Object o, Object[] oa) {
        for (Object ob : oa) {
            if (ob.equals(o)) {
                return true;
            }
        }

        return false;
    }

    private HashMap<String, Object> getAttributes(HttpSession session) {
        HashMap<String, Object> list = new HashMap<String, Object>();
        Enumeration<String> attributeNames = session.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            String name = attributeNames.nextElement();
            list.put(name, session.getAttribute(name));
        }

        return list;
    }

    private HttpSession addMapToSession(HashMap<String, Object> map, HttpSession session) {
        for (String s : map.keySet()) {
            session.setAttribute(s, map.get(s));
        }

        return session;
    }

    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException, ClassNotFoundException, InstantiationException, IllegalAccessException, IllegalArgumentException {
        response.setContentType("text/html;charset=ISO-8859-1");
        PrintWriter out = response.getWriter();

        HttpSession session = request.getSession();
        String site = (String) request.getParameter("site");
        
        try {
            Controller.userDatabase = new UserDatabase("userdb", session);
            Controller.offerDatabase = new OfferDatabase("offerdb", session);
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SAXException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (TransformerConfigurationException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (TransformerException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        }

        String[] allowedSites = {"home", "register", "logout", "create_offer"};
        String[] allowedOnlyLoggedIn = {"create_offer"};

        if (!this.isInArray(site, allowedSites)) {
            site = "home";
        }

        if (this.isInArray(site, allowedOnlyLoggedIn) && session.getAttribute("user_login") == null) {
            site = "home";
        }

        request.setAttribute("site", site);
        String siteClassString = site.substring(0, 1).toUpperCase() + site.substring(1, site.length()).replace("_", "");
        String classString = "Model." + siteClassString + "Model";
        try {
            Class c = Class.forName(classString);
            Model model = (Model) c.getDeclaredConstructors()[0].newInstance(this.getAttributes(session), request.getParameterMap(), request);
            HashMap<String, Object> map = model.run();
            session = this.addMapToSession(map, session);
            request = model.getRequest();

            if (request.getAttribute("remove_login") != null) {
                session.removeAttribute("user_login");
                request.removeAttribute("remove_login");

                request.setAttribute("site", "home");
            }

            session.setMaxInactiveInterval(30 * 60);
        } catch (ClassNotFoundException e) {
        } catch (InvocationTargetException ie) {
        }

        try {
            String url = "/WEB-INF/view/index.jsp";
            ServletContext sc = this.getServletContext();
            RequestDispatcher rd = sc.getRequestDispatcher(url);
            session.setAttribute("dispatcher", rd);
            rd.forward(request, response);
        } finally {
            out.close();
        }
    }

    public static boolean isNumeric(String str) {
        try {
            double d = Double.parseDouble(str);
        } catch (NumberFormatException nfe) {
            return false;
        }
        return true;
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        try {
            processRequest(request, response);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        try {
            processRequest(request, response);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
