package Model;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletRequest;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

/**
 *
 * @author Yannic Schnetz
 */
public class CreateofferModel extends Model {
    
    public CreateofferModel(HashMap<String, Object> attributes, Map<String, String[]> parameters, HttpServletRequest request) {
        super(attributes, parameters, request);
    }

    @Override
    public HashMap<String, Object> run() {
        if(this.getParameters().containsKey("offer_check")) {
            this.checkOffer();
        } else if(this.getParameters().containsKey("offer_send")) {
            try {
                this.createOffer();
            } catch (TransformerConfigurationException ex) {
                Logger.getLogger(CreateofferModel.class.getName()).log(Level.SEVERE, null, ex);
            } catch (TransformerException ex) {
                Logger.getLogger(CreateofferModel.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else if(this.getParameters().containsKey("offer_back")) {
            this.getBack();
        }
        
        return this.getSessionData();
    }
    
    private void getBack() {
        String name = this.getParameters().get("offer_name")[0].toString().trim();
        String mobile = this.getParameters().get("offer_mobile")[0].toString().trim();
        String description = this.getParameters().get("offer_description")[0].toString();
        String type = this.getParameters().get("offer_type")[0].toString();
        
        this.getRequest().setAttribute("offer_name", name);
        this.getRequest().setAttribute("offer_mobile", mobile);
        this.getRequest().setAttribute("offer_description", description);
        this.getRequest().setAttribute("offer_type", type);
    }
    
    private void checkOffer() {
        String name = this.getParameters().get("offer_name")[0].toString().trim();
        String mobile = this.getParameters().get("offer_mobile")[0].toString().trim();
        String description = this.getParameters().get("offer_description")[0].toString();
        String type = this.getParameters().get("offer_type")[0].toString();
        
        this.getRequest().setAttribute("offer_name", name);
        this.getRequest().setAttribute("offer_mobile", mobile);
        this.getRequest().setAttribute("offer_description", description);
        this.getRequest().setAttribute("offer_type", type);
        
        if(name.isEmpty() || mobile.isEmpty() || description.isEmpty() || type.isEmpty()) {
            this.getRequest().setAttribute("formError", "Du hast nicht alle Daten angegeben!");
            return;
        }
        
        if(name.length() < 5) {
            this.getRequest().setAttribute("formError", "Der Name muss mindestens 5 Zeichen beinhalten!");
            return;
        }
        
        if(mobile.length() < 10) {
            this.getRequest().setAttribute("formError", "Die HandyNr. muss numerisch sein und muss minimal 10 Zeichen beinhalten!");
            return;
        }
        
        if(description.length() < 50) {
            this.getRequest().setAttribute("formError", "Die Beschreibung muss länger als 50 Zeichen sein!");
            return;
        }
        
        if( ! type.equals("search") && ! type.equals("offer")) {
            this.getRequest().setAttribute("formError", "Fehler bei der Typ-Auswahl!");
            return;
        }
        
        if( ! Controller.isNumeric(mobile)) {
            this.getRequest().setAttribute("formError", "Die Handy-Nummer darf nur aus Zahlen bestehen!");
            return;
        }

        this.getRequest().setAttribute("offer_check_correct", "true");
        this.getRequest().setAttribute("formSuccess", "Alle Eingaben sind korrekt, du kannst das Angebot mit einem Klick auf 'Abschicken' erstellen!");
    }
    
    private void createOffer() throws TransformerConfigurationException, TransformerException {
        String name = this.getParameters().get("offer_name")[0].toString().trim();
        String mobile = this.getParameters().get("offer_mobile")[0].toString().trim();
        String description = this.getParameters().get("offer_description")[0].toString();
        String type = this.getParameters().get("offer_type")[0].toString();
        
        Controller.offerDatabase.saveOffer(name, mobile, description, type, (User) this.getAttributes().get("user_login"));
        this.getRequest().setAttribute("formSuccess", "Angebot erfolgreich erstellt!");
    }
}
