package Model;

import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *
 * @author Yannic Schnetz
 */
public abstract class Model {
    
    private HashMap<String, Object> attributes = null;
    private Map<String, String[]> parameters = null;
    private HashMap<String, Object> sessionData = null;
    private HttpServletRequest request = null;
    
    public abstract HashMap<String, Object> run();
    
    public Model(HashMap<String, Object> attributes, Map<String, String[]> parameters, HttpServletRequest request) {
        this.attributes = attributes;
        this.parameters = parameters;
        this.request = request;
        this.sessionData = new HashMap<String, Object>();
        
        if(this.parameters.containsKey("submit_login")) {
            try {
                this.loginUser();
            } catch (NoSuchAlgorithmException ex) {
                Logger.getLogger(HomeModel.class.getName()).log(Level.SEVERE, null, ex);
            } catch (UnsupportedEncodingException ex) {
                Logger.getLogger(HomeModel.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    
    public static void locate(HttpServletResponse res, String url) {
        res.setStatus(res.SC_MOVED_TEMPORARILY);
        res.setHeader("Location", url);
    }
    
    private void loginUser() throws NoSuchAlgorithmException, UnsupportedEncodingException {
        String email = this.getParameters().get("textbox_mail")[0].toString().trim().toLowerCase();
        String password = this.getParameters().get("password_login")[0].toString();
        
        if(email.isEmpty() || password.isEmpty()) {
            this.getRequest().setAttribute("loginError", "Du hast nicht alle Felder angegeben!");
            return;
        }
        
        if( ! Controller.userDatabase.userExists(email)) {
            this.getRequest().setAttribute("loginError", "Diese E-Mail Adresse ist nicht registriert!");
            return;
        }
        
        password = User.decryptPassword(password, email);
        
        String accPass = Controller.userDatabase.getData("password", false, "email", email).toString();
        
        if( ! accPass.equals(password)) {
            this.getRequest().setAttribute("loginError", "Das Passwort stimmt nicht mit der E-Mail Adresse überein!");
            return;
        }
        
        User user = new User();
        user.setEmail(email);
        user.setId(Integer.parseInt(Controller.userDatabase.getData("id", true, "email", email).toString()));
        user.setPassword(password);
        
        String date = Controller.userDatabase.getData("creationDate", false, "email", email).toString();
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        try {
            user.setCreationDate(dateFormat.parse(date));
        } catch (ParseException ex) {
            this.getRequest().setAttribute("loginError", "Fehler beim Formatieren des Datums!");
            return;
        }
        
        this.getSessionData().put("user_login", user);
    }

    public HashMap<String, Object> getAttributes() {
        return attributes;
    }

    public Map<String, String[]> getParameters() {
        return parameters;
    }

    public HashMap<String, Object> getSessionData() {
        return sessionData;
    }

    public HttpServletRequest getRequest() {
        return request;
    }
    
}
