package Model;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 *
 * @author Yannic Schnetz
 */
public class OfferDatabase extends Database {
    
    public OfferDatabase(String name, HttpSession session) throws ParserConfigurationException, SAXException, IOException, TransformerConfigurationException, TransformerException {
        super(name, session);
    }

    @Override
    public Object getData(String nodeName, boolean attribute, String attributeName, String attributeValue) {
        try {
            String expression = "";
            
            if (!attributeName.equals("") && !attributeValue.equals("")) {
                if(attribute) {
                    expression = "/" + this.getFilename() + "/user[@" + attributeName + "='" + attributeValue + "']/@" + nodeName;
                } else {
                    expression = "/" + this.getFilename() + "/user[@" + attributeName + "='" + attributeValue + "']/" + nodeName;
                }
                
                return this.getxPath().compile(expression).evaluate(this.getXml());
            } else {
                expression = "/" + this.getFilename() + "/" + nodeName;
                
                return (NodeList) this.getxPath().compile(expression).evaluate(this.getXml(), XPathConstants.NODESET);
            }

        } catch (XPathExpressionException ex) {
            Logger.getLogger(UserDatabase.class.getName()).log(Level.SEVERE, null, ex);
        }

        return null;
    }
    
    public ArrayList<Offer> getOffers(int limit) throws XPathExpressionException {
        String expression = "/" + this.getFilename() + "/offer";
        ArrayList<Offer> offerlist = new ArrayList<Offer>();
        
        NodeList offers = (NodeList) this.getxPath().compile(expression).evaluate(this.getXml(), XPathConstants.NODESET);
        
        for(int i = 0; i < offers.getLength(); i++) {
            String type = offers.item(i).getAttributes().getNamedItem("type").getNodeValue();
            String id = offers.item(i).getAttributes().getNamedItem("id").getNodeValue();
            String userid = offers.item(i).getAttributes().getNamedItem("user_id").getNodeValue();
            String exp = "/" + this.getFilename() + "/offer[@id='" + id + "']/name";
            
            String name = this.getxPath().compile(exp).evaluate(this.getXml()).toString();
            
            exp = "/" + this.getFilename() + "/offer[@id='" + id + "']/mobile";
            String mobile = this.getxPath().compile(exp).evaluate(this.getXml()).toString();
            
            exp = "/" + this.getFilename() + "/offer[@id='" + id + "']/description";
            String description = this.getxPath().compile(exp).evaluate(this.getXml()).toString();
            
            Offer o = new Offer();
            o.setId(Integer.parseInt(id));
            o.setDescription(description);
            o.setMobile(mobile);
            o.setName(name);
            o.setType(type);
            o.setUser(Controller.userDatabase.getUser(userid));
            
            offerlist.add(o);
        }
        
        return offerlist;
    }
    
    
    public void saveOffer(String name, String mobile, String description, String type, User user) throws TransformerConfigurationException, TransformerException {
        Element database = this.getXml().getDocumentElement();
        int lastid = 0;

        NodeList users = this.getXml().getElementsByTagName("offer");
        if (users.getLength() > 0) {
            lastid = Integer.parseInt(users.item(users.getLength() - 1).getAttributes().item(0).getNodeValue());
        }

        Element offerNode = this.getXml().createElement("offer");
        database.appendChild(offerNode);

        offerNode.setAttribute("id", Integer.toString(lastid + 1));
        offerNode.setAttribute("user_id", Integer.toString(user.getId()));
        offerNode.setAttribute("type", type);

        Element offername = this.getXml().createElement("name");
        offername.appendChild(this.getXml().createTextNode(name));
        offerNode.appendChild(offername);
        
        Element offermobile = this.getXml().createElement("mobile");
        offermobile.appendChild(this.getXml().createTextNode(mobile));
        offerNode.appendChild(offermobile);
        
        Element offerdescription = this.getXml().createElement("description");
        offerdescription.appendChild(this.getXml().createTextNode(description));
        offerNode.appendChild(offerdescription);

        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm");
        Date date = new Date();

        Element creation = this.getXml().createElement("creationDate");
        creation.appendChild(this.getXml().createTextNode(dateFormat.format(date))); // creation.setNodeValue(dateFormat.format(date));
        offerNode.appendChild(creation);

        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        DOMSource source = new DOMSource(this.getXml());
        StreamResult result = new StreamResult(new File(this.getSession().getServletContext().getRealPath("") + "/xml/" + this.getFilename() + ".xml"));

        transformer.transform(source, result);
    }
}
