package Model;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import nl.captcha.Captcha;
import java.security.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletRequest;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;


/**
 *
 * @author Yannic Schnetz
 */
public class RegisterModel extends Model {
    
    public RegisterModel(HashMap<String, Object> attributes, Map<String, String[]> parameters, HttpServletRequest request) {
        super(attributes, parameters, request);
    }

    @Override
    public HashMap<String, Object> run() {
        if(this.getParameters().containsKey("register_submit")) {
            try {
                this.registerUser();
            } catch (UnsupportedEncodingException ex) {
                Logger.getLogger(RegisterModel.class.getName()).log(Level.SEVERE, null, ex);
            } catch (NoSuchAlgorithmException ex) {
                Logger.getLogger(RegisterModel.class.getName()).log(Level.SEVERE, null, ex);
            } catch (TransformerConfigurationException ex) {
                Logger.getLogger(RegisterModel.class.getName()).log(Level.SEVERE, null, ex);
            } catch (TransformerException ex) {
                Logger.getLogger(RegisterModel.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        
        return this.getSessionData();
    }
    
    public void registerUser() throws UnsupportedEncodingException, NoSuchAlgorithmException, TransformerConfigurationException, TransformerException {
        String email = this.getParameters().get("register_email")[0].toString().trim().toLowerCase();
        String password = this.getParameters().get("register_password")[0].toString();
        String password_wdh = this.getParameters().get("register_password_wdh")[0].toString();
        String captcha = this.getParameters().get("register_captcha")[0].toString().trim();
        
        this.getRequest().setAttribute("register_email", email);
        
        Pattern p = Pattern.compile("^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
		+ "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$");
        Matcher m = p.matcher(email);
        boolean matchFound = m.matches();
        
        if(email.isEmpty() || password.isEmpty() || password_wdh.isEmpty() || captcha.isEmpty()) {
            this.getRequest().setAttribute("formError", "Du hast nicht alle notwendigen Felder angegeben!");
            return;
        }
        
        if(password.length() < 3 || password.length() > 30) {
            this.getRequest().setAttribute("formError", "Das Passwort muss zwischen 3 und 30 Zeichen haben!");
            return;
        }
        
        if( ! matchFound) {
            this.getRequest().setAttribute("formError", "Du musst eine gültige Mail-Adresse angeben!");
            return;
        }
        
        if( ! password.equals(password_wdh)) {
            this.getRequest().setAttribute("formError", "Dein Passwort und dessen Wiederholung stimmt nicht überein!");
            return;
        }
        
        Captcha cap = (Captcha) this.getAttributes().get("CAPTCHA");
        if( ! cap.isCorrect(captcha)) {
            this.getRequest().setAttribute("formError", "Das Captcha wurde leider falsch eingegeben!");
            return;
        }
        
        if(Controller.userDatabase.userExists(email)) {
            this.getRequest().setAttribute("formError", "Diese Mail-Adresse wurde bereits registriert!");
            return;
        }
        
        password = User.decryptPassword(password, email);
        
        Controller.userDatabase.saveUser(User.createUser(email, password));
        this.getSessionData().remove("register_email");
        this.getRequest().setAttribute("formSuccess", "Dein Account wurde erfolgreich erstellt! Du kannst dich nun einloggen.");
    }
}
