package Model;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 *
 * @author Yannic Schnetz
 */
public class UserDatabase extends Database {

    public UserDatabase(String name, HttpSession session) throws ParserConfigurationException, SAXException, IOException, TransformerConfigurationException, TransformerException {
        super(name, session);
    }
    
    public User getUser(String userid) {
        String email = this.getData("email", true, "id", userid).toString();
        String password = this.getData("password", false, "id", userid).toString();
        int id = Integer.parseInt(userid);
        String creation = this.getData("creationDate", false, "id", userid).toString();
        
        String[] vals = creation.split("-");
        
        User user = new User();
        user.setCreationDate(new Date(Integer.parseInt(vals[0]), Integer.parseInt(vals[1]), Integer.parseInt(vals[2])));
        user.setEmail(email);
        user.setPassword(password);
        user.setId(id);
        
        return user;
    }

    public void saveUser(User user) throws TransformerConfigurationException, TransformerException {
        Element database = this.getXml().getDocumentElement();
        int lastid = 0;

        NodeList users = this.getXml().getElementsByTagName("user");
        if (users.getLength() > 0) {
            lastid = Integer.parseInt(users.item(users.getLength() - 1).getAttributes().item(1).getNodeValue());
        }

        Element userNode = this.getXml().createElement("user");
        database.appendChild(userNode);

        userNode.setAttribute("id", Integer.toString(lastid + 1));
        userNode.setAttribute("email", user.getEmail());

        Element password = this.getXml().createElement("password");
        password.appendChild(this.getXml().createTextNode(user.getPassword()));
        userNode.appendChild(password);

        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();

        Element creation = this.getXml().createElement("creationDate");
        creation.appendChild(this.getXml().createTextNode(dateFormat.format(date))); // creation.setNodeValue(dateFormat.format(date));
        userNode.appendChild(creation);

        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        DOMSource source = new DOMSource(this.getXml());
        StreamResult result = new StreamResult(new File(this.getSession().getServletContext().getRealPath("") + "/xml/" + this.getFilename() + ".xml"));

        transformer.transform(source, result);
    }

    public boolean userExists(String mail) {
        Object data = this.getData("email", true, "email", mail);
        if(data != null && ! data.equals("")) {
            return true;
        }

        return false;
    }

    @Override
    public Object getData(String nodeName, boolean attribute, String attributeName, String attributeValue) {
        try {
            String expression = "";
            
            if (!attributeName.equals("") && !attributeValue.equals("")) {
                if(attribute) {
                    expression = "/userdb/user[@" + attributeName + "='" + attributeValue + "']/@" + nodeName;
                } else {
                    expression = "/userdb/user[@" + attributeName + "='" + attributeValue + "']/" + nodeName;
                }
                
                return this.getxPath().compile(expression).evaluate(this.getXml());
            } else {
                expression = "/userdb/" + nodeName;
                
                return (NodeList) this.getxPath().compile(expression).evaluate(this.getXml(), XPathConstants.NODESET);
            }

        } catch (XPathExpressionException ex) {
            Logger.getLogger(UserDatabase.class.getName()).log(Level.SEVERE, null, ex);
        }

        return null;
    }
}
